"use strict";

import * as TextPopupManager from "../TextPopup/TextPopupManager.js";
import * as StageContainer   from "../../Stages/StageContainer.js";
import * as GravityManager   from "./../GravityManager.js";
import * as SoundManager     from "../SoundManager.js";
import * as GlobalData       from "../../Global/GlobalData.js";
import * as GameField        from "../../Global/GameField.js";

import * as Tools            from "./../Tools.js";
import { class_Explosion } from "./Class_Explosion.js";

export class class_ExplosionCluster
{
	#explosionMap = new Map();
	#points = 0;

	#rowPrimary;
  #colPrimary;
	#tileLocationSet;

	constructor(rowPrimary, colPrimary, tileLocationSet)
	{
		this.#rowPrimary = rowPrimary;
		this.#colPrimary = colPrimary;
		this.#tileLocationSet = tileLocationSet;

		this.#createExplosions();
		this.#calculatePoints();
	}

	#createExplosions()
	{
		SoundManager.playSoundByIndex(8);

		for (const tileLocation of this.#tileLocationSet.values())
		{
			const rowAndColArray  = Tools.getRowAndColArray(tileLocation);
			const explosionNumber = Tools.getNewExplosionNumber();

			const explosion = new class_Explosion(rowAndColArray, explosionNumber);
			this.#explosionMap.set(explosionNumber, explosion);
		}
	}

	#calculatePoints()
	{
		this.#points = (this.#explosionMap.size * this.#explosionMap.size * 10);
	}

	updateState()
	{
		const explosionsToDelete = [];

		for (const key of this.#explosionMap.keys())
		{
			const explosion = this.#explosionMap.get(key);
			if (!explosion.isAnimationFinished)
			{
				explosion.updateState();
			}
			else
			{
				explosionsToDelete.push(key);
			}
		}

		for (let i = 0; i < explosionsToDelete.length; i++)
		{
			const key = explosionsToDelete[i];
			const explosion = this.#explosionMap.get(key);

			GameField.setTile(explosion.getRow(), explosion.getCol(), GlobalData.enumTileNumber.NO_TILE); // This is needed to trigger gravity.

			GravityManager.addCol(explosion.getCol());
			this.#explosionMap.delete(key);
		}

		if (this.isClusterFinished())
		{
			this.#createPointsPopup();
			StageContainer.addPointsToCurrentStage(this.#points);
			this.#points = 0;
		}
	}

	isClusterFinished()
	{
		return (this.#explosionMap.size === 0);
	}

	#createPointsPopup()
	{
		const X = Tools.getTileX(this.#colPrimary);
		const Y = Tools.getTileY(this.#rowPrimary) + 32;
		TextPopupManager.addText(this.#points, X, Y);
	}

	updateExplosionWithNewLocations(newExplosionLocationsMap)
	{
		for (let [key, value] of newExplosionLocationsMap.entries())
		{
			if (this.#explosionMap.has(key))
			{
				const explosion = this.#explosionMap.get(key);
				const rowAndColArray = Tools.getRowAndColArray(value);
				explosion.setRow(rowAndColArray[0]);
				explosion.setCol(rowAndColArray[1]);
			}
		}
	}

	render()
	{
		for (const key of this.#explosionMap.keys())
		{
			const explosion = this.#explosionMap.get(key);
			explosion.render();
		}
	}

}
